@@ -37,28 +37,28 @@ use constant MEDIATYPE_PODCAST_VIDEO => 6;
 use constant MACTIME => GNUpod::FooBar::MACTIME;
 use vars qw(%opts %dupdb_normal %dupdb_lazy %dupdb_podcast $int_count %podcast_infos %podcast_channel_infos %per_file_info);
 
-print "gnupod_addsong.pl Version 0.99.7 (C) Adrian Ulrich\n";
+print "gnupod_addsong.pl Version 0.99.8 (C) Adrian Ulrich\n";
 
 $int_count = 3; #The user has to send INT (Ctrl+C) x times until we stop
 
 $opts{mount} = $ENV{IPOD_MOUNTPOINT};
 #Don't add xml and itunes opts.. we *NEED* the mount opt to be set..
 GetOptions(\%opts, "version", "help|h", "mount|m=s", "decode|x=s", "restore|r", "duplicate|d", "disable-v2", "disable-v1",
-                   "set-title|t=s", "set-artist|a=s", "set-album|l=s", "set-genre|g=s", "set-rating=i", "set-playcount=i",
-                   "set-bookmarkable|b", "set-shuffleskip", "artwork=s",
+                   "disable-ape-tag", "set-title|t=s", "set-artist|a=s", "set-album|l=s", "set-genre|g=s", "set-rating=i",
+                   "set-playcount=i", "set-bookmarkable|b", "set-shuffleskip", "artwork=s", "replaygain-album",
                    "set-songnum", "playlist|p=s@", "reencode|e=i",
                    "min-vol-adj=i", "max-vol-adj=i", "playlist-is-podcast", "podcast-files-limit=i", "podcast-cache-dir=s",
                    "podcast-artwork", "set-compilation");
 
-GNUpod::FooBar::GetConfig(\%opts, {'decode'=>'s', mount=>'s', duplicate=>'b', model=>'s',
-                                   'disable-v1'=>'b', 'disable-v2'=>'b', 'set-songnum'=>'b',
+GNUpod::FooBar::GetConfig(\%opts, {'decode'=>'s', mount=>'s', duplicate=>'b', model=>'s', 'replaygain-album'=>'b',
+                                   'disable-v1'=>'b', 'disable-v2'=>'b', 'disable-ape-tag'=>'b', 'set-songnum'=>'b',
                                    'min-vol-adj'=>'i', 'max-vol-adj'=>'i', 'automktunes'=>'b', 
                                    'podcast-files-limit'=>'i', 'podcast-cache-dir'=>'s', 'podcast-artwork'=>'b' },
                                    "gnupod_addsong");
 
 
 
-usage("\n--decode needs 'pcm' 'mp3' 'aac' 'video' or 'aacbm' -> '--decode=mp3'\n") if $opts{decode} && $opts{decode} !~ /^(mp3|video|aac|aacbm|pcm|crashme)$/;
+usage("\n--decode needs 'pcm' 'mp3' 'aac' 'video' 'aacbm' or 'alac' -> '--decode=mp3'\n") if $opts{decode} && $opts{decode} !~ /^(mp3|video|aac|aacbm|pcm|alac|crashme)$/;
 usage()   if $opts{help};
 version() if $opts{version};
 
@@ -122,14 +122,6 @@ sub startup {
 		GNUpod::XMLhelper::doxml($con->{xml}) or usage("Failed to parse $con->{xml}, did you run gnupod_INIT.pl?\n");
 	}
 	
-	if($opts{playlist}) { #Create this playlist
-		foreach my $xcpl (@{$opts{playlist}}) {
-			print "> Adding songs to Playlist '$xcpl'\n";
-			GNUpod::XMLhelper::addpl($xcpl, {podcast=>$opts{'playlist-is-podcast'}}); #Fixme: this may printout a warning..
-		}
-	}
-	
-	
 	# Check volume adjustment options for sanity
 	my $min_vol_adj = int($opts{'min-vol-adj'});
 	my $max_vol_adj = int($opts{'max-vol-adj'});
@@ -144,6 +136,17 @@ sub startup {
 	my @real_files = resolve_podcasts(@argv_files);
 	my $addcount   = 0;
 	
+	if($opts{playlist}) { #Create this playlist
+		foreach my $xcpl (@{$opts{playlist}}) {
+			print "> Adding songs to Playlist '$xcpl'\n";
+			GNUpod::XMLhelper::addpl($xcpl, {podcast=>$opts{'playlist-is-podcast'}}); #Fixme: this may printout a warning..
+		}
+	} elsif ($opts{'playlist-is-podcast'} && (scalar(@real_files) > 0)) {
+		print "> Adding songs to Playlist '$per_file_info{$real_files[0]}->{album}'\n";
+		push @{$opts{playlist}}, $per_file_info{$real_files[0]}->{album};
+		GNUpod::XMLhelper::addpl($per_file_info{$real_files[0]}->{album}, {podcast=>$opts{'playlist-is-podcast'}}); #Fixme: this may printout a warning..
+	}
+	
 	#We are ready to copy each file..
 	foreach my $file (@real_files) {
 		#Skip all songs if user sent INT
@@ -154,6 +157,8 @@ sub startup {
 		#Get the filetype
 		my ($fh,$media_h,$converter) =  GNUpod::FileMagic::wtf_is($file, {noIDv1=>$opts{'disable-v1'}, 
 		                                                                  noIDv2=>$opts{'disable-v2'},
+		                                                                  noAPE=>$opts{'disable-ape-tag'},
+		                                                                  rgalbum=>$opts{'replaygain-album'},
 		                                                                  decode=>$opts{'decode'}},$con);
 		
 		unless($fh) {
@@ -170,7 +175,11 @@ sub startup {
 		my $c_per_file_info = $per_file_info{$file};
 		foreach(keys(%$c_per_file_info)) {
 			next unless lc($_) eq $_; #lc keys are there to overwrite $fh keys
-			$fh->{$_} = $c_per_file_info->{$_};
+			if ($_ eq "desc") {
+				$fh->{$_} = GNUpod::FileMagic::__merge_strings({joinby => "\n", wspace => "norm", case => "ignore" }, $c_per_file_info->{$_}, $fh->{$_});
+			} else {
+				$fh->{$_} = $c_per_file_info->{$_};
+			}
 		}
 		$c_per_file_info->{ISPODCAST} ||= $opts{'playlist-is-podcast'};  # Enforce podcast settings if we are going to create a pc-playlist
 		
@@ -194,7 +203,8 @@ sub startup {
 		
 		#Set the addtime to unixtime(now)+MACTIME (the iPod uses mactime)
 		#This breaks perl < 5.8 if we don't use int(time()) !
-		$fh->{addtime} = int(time())+MACTIME;
+		#Use fixed addtime for autotests
+		$fh->{addtime} = int($connection->{autotest} ? 42 : time())+MACTIME;
 		
 		#Ugly workaround to avoid a warning while running mktunes.pl:
 		#All (?) int-values returned by wtf_is won't go above 0xffffffff
@@ -569,6 +579,7 @@ sub resolve_podcasts {
 			
 			unless($env_is_okay) {
 				warn "! [!!!!] WARNING: This podcast may not appear on your iPod because you did not specify a podcast-playlist to use.\n";
+				warn "! [!!!!]          I will try to use the podcast's title as a playlist name but there's no guarantee it will work out.\n";
 				warn "! [!!!!]          Please use the options '--playlist' and '--playlist-is-podcast' while fetching podcasts.\n";
 			}
 			
@@ -584,7 +595,14 @@ sub resolve_podcasts {
 				my $py = new XML::Parser(Handlers=>{Start=>\&podcastChannelStart, Char=>\&podcastChannelChar, End=>\&podcastChannelEnd});
 				$py->parsefile($pcrss->{file});
 			};
-			warn "! [HTTP] Error while parsing XML: $@\n" if $@;
+			if ($@) {
+				warn "! [HTTP] Error while parsing XML file ".$pcrss->{file}.". File kept for examination. : $@\n";
+				next;
+			}
+			if (!defined($podcast_infos{$pcrss->{file}})) {
+				warn "! [HTTP] No item found while parsing XML file ".$pcrss->{file}.". File kept for examination.\n";
+				next;
+			}
 			unlink($pcrss->{file}) or warn "Could not unlink $pcrss->{file}, $!\n";
 
 			#Limit the number of podcasts to dowload.
@@ -594,7 +612,7 @@ sub resolve_podcasts {
 				splice(@pods, $flimit);
 			} elsif ($flimit < 0) {
 				splice(@pods, 0, $flimit);
-			}				
+			}
 			$podcast_infos{$pcrss->{file}} = \@pods;
 
 			$per_file_info{$pcrss->{file}}->{REAL_RSS} = $cf;
@@ -634,6 +652,7 @@ sub resolve_podcasts {
 					 $podcast_item->{author}->{"\0"} or
 					 $channel->{"itunes:author"}->{"\0"} or
 					 $channel->{"managingEditor"}->{"\0"} );
+			my $c_album = $channel->{"title"}->{"\0"};
 			my $c_rdate = $podcast_item->{pubDate}->{"\0"}; 
 			my $c_desc  = $podcast_item->{description}->{"\0"};
 			my $c_url   = $podcast_item->{enclosure}->{url};
@@ -664,6 +683,7 @@ sub resolve_podcasts {
 			$per_file_info{$rssmedia->{file}}->{podcastrss}  = $c_podcastrss;
 			$per_file_info{$rssmedia->{file}}->{title}       = $c_title   if $c_title;
 			$per_file_info{$rssmedia->{file}}->{artist}      = $c_author  if $c_author;
+			$per_file_info{$rssmedia->{file}}->{album}       = $c_album   if $c_album;
 			$per_file_info{$rssmedia->{file}}->{desc}        = $c_desc    if $c_desc;
 			$per_file_info{$rssmedia->{file}}->{releasedate} = int(Date::Parse::str2time($c_rdate))+MACTIME    if $c_rdate;
 			
@@ -723,8 +743,11 @@ Usage: gnupod_addsong.pl [-h] [-m directory] File1 File2 ...
                                     0 = download all (default), -X = download X oldest items, X = download X newest items
        --disable-v1                 Do not read ID3v1 Tags (MP3 Only)
        --disable-v2                 Do not read ID3v2 Tags (MP3 Only)
+       --disable-ape-tag            Do not read APE Tags (MP3 Only) --disable-v1 with --disable-v2 implies --disable-ape-tag
+       --replaygain-album           Use the ReplayGain album value instead of the ReplayGain track value (default)
    -x  --decode=pcm|mp3|aac|aacbm   Convert FLAC Files to WAVE/MP3 or AAC 'on-the-fly'. Use '-e' to specify a quality/bitrate
    -x  --decode=video               Convert .avi Files into iPod video 'on-the-fly' (needs ffmpeg with AAC support)
+   -x  --decode=alac                Convert FLAC Files into Apple Lossless 'on-the-fly' (needs ffmpeg with ALAC support)
    -e  --reencode=int               Reencode MP3/AAC files with new quality 'on-the-fly'
                                     (0 = Good .. 9 = Bad)
                                     You may be able to save some space if you do not need
@@ -739,10 +762,10 @@ Usage: gnupod_addsong.pl [-h] [-m directory] File1 File2 ...
    -b  --set-bookmarkable           Set this song as bookmarkable (= Remember position)
        --set-shuffleskip            Exclude this file in shuffle-mode
        --set-compilation            Mark songs as being part of a compilation
-       --min-vol-adj=int            Minimum volume adjustment allowed by ID3v2.4 RVA2 tag
-       --max-vol-adj=int            Maximum ditto.  The volume can be adjusted in the range
-                                    -100% to +100%.  The default for these two options is 0,
-                                    which effectively ignored the RVA2 tag.
+       --min-vol-adj=int            Minimum volume adjustment allowed by ID3 RVA/RVAD tag
+       --max-vol-adj=int            Maximum ditto. The volume can be adjusted manually in iTunes in
+                                    the range -100% to +100%. The default for these two options is 0,
+                                    which effectively ignores the RVA/RVAD tag.
        --artwork=FILE               Use FILE as album cover
 
 
@@ -752,7 +775,7 @@ EOF
 
 sub version {
 die << "EOF";
-gnupod_addsong.pl (gnupod) 0.99.7
+gnupod_addsong.pl (gnupod) 0.99.8
 Copyright (C) Adrian Ulrich 2002-2008
 
 This is free software; see the source for copying conditions.  There is NO
